﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace TradingMachineSample
{
    public abstract class TradingMachineManager<T> where T: TradingMachine, new()
    {
        static protected ObjectManager<T> s_TMDict = new ObjectManager<T>();

        #region C++ interface...

        public static int Init(ref int a_errorCode)
        {
            try
            {
                StartProcessing(ref a_errorCode);
                T tm = new T();
                return s_TMDict.RegisterObject(tm);
            }
            catch (Exception exc)
            {
                ExceptionProcessing(exc, ref a_errorCode);
                return 0;
            }
        }

        public static void DeInit(int a_index, ref int a_errorCode)
        {
            try
            {
                StartProcessing(ref a_errorCode);
                s_TMDict.UnregisterObject(a_index);
            }
            catch (Exception exc)
            {
                ExceptionProcessing(exc, ref a_errorCode);
            }
        }

        public static void Start(string a_symbol, double a_ask, double a_bid, int a_index,
            ref int a_errorCode)
        {
            try
            {
                StartProcessing(ref a_errorCode);
                s_TMDict[a_index].Start(a_symbol, a_ask, a_bid);
            }
            catch (Exception exc)
            {
                ExceptionProcessing(exc, ref a_errorCode);
            }
        }

        #endregion

        #region Private section...

        protected static void StartProcessing(ref int a_errorCode)
        {
            a_errorCode = 0;
        }

        protected static void ExceptionProcessing(Exception a_exc, ref int a_errorCode)
        {
            Logging.WriteLog(a_exc);
            a_errorCode = -1;
        }

        #endregion
    }

    public abstract class TradingMachine
    {
        public abstract void Start(string a_symbol, double a_ask, double a_bid);
    }

    public class TMSampleManager: TradingMachineManager<TMSample>
    {
        #region C++ interface...

        public static int GetIntValue(int a_index, ref int a_errorCode)
        {
            try
            {
                StartProcessing(ref a_errorCode);
                return s_TMDict[a_index].GetIntValue();
            }
            catch (Exception exc)
            {
                ExceptionProcessing(exc, ref a_errorCode);
                return 0;
            }
        }

        public static string GetStringValue(int a_index, ref int a_errorCode)
        {
            try
            {
                StartProcessing(ref a_errorCode);
                return s_TMDict[a_index].GetStringValue();
            }
            catch (Exception exc)
            {
                ExceptionProcessing(exc, ref a_errorCode);
                return "";
            }
        }

        #endregion
    }

    public class TMSample: TradingMachine
    {
        double m_ask = 0;
        double m_bid = 0;

        public override void Start(string a_symbol, double a_ask, double a_bid)
        {
            m_ask = a_ask;
            m_bid = a_bid;
        }

        public int GetIntValue()
        {
            return 4;
        }

        public string GetStringValue()
        {
            return "Result string. Результирующая строка.";
        }
    }
}
